influxcloud

package module
v0.0.0-...-592d533 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: May 5, 2017 License: MIT Imports: 6 Imported by: 3

README

influxCloud

Go Report Card Build Status

Status

influx-meta is ready to go. But this need corporate with influxd. Comparing with current influxd, we have to append a cluster service. In this way, It enables the communication channel between data node and meta node.

What is this prototype can do?

  • edit meta node including adding, updating and deleting
  • edit data node including adding, updating and deleting
  • distributed data in a basic form of shards across nodes in cluster
  • distributed query across cluster if such query can not be done locally

What is this prototype can not do but will add in future?

  • Raft Algorithm Optimization
  • Performance improvement
  • Provide a way expand Shades inside ShardGroup. For now, once ShardGroup is created, there is no way that we can expand the size of Shards which means the capacity of writing will be limited.
  • Add User and Role Creation and Authorization.
  • Added Command that can know the status of cluster
  • Bring docker into build
  • add integration framework based on etcd's work.
  • Buffer failed write into disk and retry until such buffer is empty. In order to improve the usage of disk, we need clean such buffer in some manner.
  • Provide a way that can backup a cluster and restore it later.

How to build

Well, you do not need worry this in a month. The prototype is still under implementing. But we promise, we will try hard to get things done quickly.

Architeture

Please check the architeture.md for more details. Any inquiry and comments are welcome. You can find my email in my github's profile.

Documentation

Overview

Package influxcloud is the root package of Influxcloud, the scalable datastore for metrics, events, and real-time analytics.

Index

Constants

This section is empty.

Variables

var ErrFieldTypeConflict = errors.New("field type conflict")

ErrFieldTypeConflict is returned when a new field already exists with a different type.

Functions

func ErrDatabaseNotFound

func ErrDatabaseNotFound(name string) error

ErrDatabaseNotFound indicates that a database operation failed on the specified database because the specified database does not exist.

func ErrRetentionPolicyNotFound

func ErrRetentionPolicyNotFound(name string) error

ErrRetentionPolicyNotFound indicates that the named retention policy could not be found in the database.

func IsClientError

func IsClientError(err error) bool

IsClientError indicates whether an error is a known client error.

Types

type Node

type Node struct {
	ID uint64
	// contains filtered or unexported fields
}

func LoadNode

func LoadNode(path string) (*Node, error)

LoadNode will load the node information from disk if present

func NewNode

func NewNode(path string) *Node

NewNode will return a new node

func (*Node) Save

func (n *Node) Save() error

Save will save the node file to disk and replace the existing one if present

Directories

Path Synopsis
cmd
influxd command
Command influxd is the InfluxDB server.
Command influxd is the InfluxDB server.
influxd-meta command
influxd/backup
Package backup is the backup subcommand for the influxd command.
Package backup is the backup subcommand for the influxd command.
influxd/help
Package help is the help subcommand of the influxd command.
Package help is the help subcommand of the influxd command.
influxd/restore
Package restore is the restore subcommand for the influxd command, for restoring from a backup.
Package restore is the restore subcommand for the influxd command, for restoring from a backup.
influxd/run
Package run is the run (default) subcommand for the influxd command.
Package run is the run (default) subcommand for the influxd command.
Package hh implements a hinted handoff for writes
Package hh implements a hinted handoff for writes
internal
Package internal is a generated protocol buffer package.
Package internal is a generated protocol buffer package.
rpc
internal
Package internal is a generated protocol buffer package.
Package internal is a generated protocol buffer package.

Jump to