Documentation
¶
Overview ¶
Package influxcloud is the root package of Influxcloud, the scalable datastore for metrics, events, and real-time analytics.
Index ¶
Constants ¶
This section is empty.
Variables ¶
var ErrFieldTypeConflict = errors.New("field type conflict")
ErrFieldTypeConflict is returned when a new field already exists with a different type.
Functions ¶
func ErrDatabaseNotFound ¶
ErrDatabaseNotFound indicates that a database operation failed on the specified database because the specified database does not exist.
func ErrRetentionPolicyNotFound ¶
ErrRetentionPolicyNotFound indicates that the named retention policy could not be found in the database.
func IsClientError ¶
IsClientError indicates whether an error is a known client error.
Types ¶
Directories
¶
| Path | Synopsis |
|---|---|
|
cmd
|
|
|
influxd
command
Command influxd is the InfluxDB server.
|
Command influxd is the InfluxDB server. |
|
influxd-meta
command
|
|
|
influxd/backup
Package backup is the backup subcommand for the influxd command.
|
Package backup is the backup subcommand for the influxd command. |
|
influxd/help
Package help is the help subcommand of the influxd command.
|
Package help is the help subcommand of the influxd command. |
|
influxd/restore
Package restore is the restore subcommand for the influxd command, for restoring from a backup.
|
Package restore is the restore subcommand for the influxd command, for restoring from a backup. |
|
influxd/run
Package run is the run (default) subcommand for the influxd command.
|
Package run is the run (default) subcommand for the influxd command. |
|
Package hh implements a hinted handoff for writes
|
Package hh implements a hinted handoff for writes |
|
internal
Package internal is a generated protocol buffer package.
|
Package internal is a generated protocol buffer package. |
|
internal
Package internal is a generated protocol buffer package.
|
Package internal is a generated protocol buffer package. |
Click to show internal directories.
Click to hide internal directories.